@@ -101,7 +101,7 @@ sub cmd_update {
 
     # reinstall everything in cpanfile to build artifacts
     my $cpanfile = $self->try_cpanfile
-      or Carp::croak "Can't locate 'cpanfile' to load module list.";
+      or die "Can't locate 'cpanfile' to load module list.\n";
     $self->install_with_cpanfile(Module::CPANfile->load($cpanfile));
 
     # then rebuild the snapshot
@@ -149,7 +149,7 @@ sub install_from_cpanfile {
             },
         });
         print "---> Installing new dependencies: ", join(", ", @missing), "\n";
-        $self->install_with_cpanfile($cpanfile);
+        $self->install_with_cpanfile($cpanfile, $no_snapshot ? () : $self->snapshot );
     }
 
     my @artifacts;
@@ -170,11 +170,20 @@ sub is_core {
 }
 
 sub install_with_cpanfile {
-    my($self, $cpanfile) = @_;
+    my($self, $cpanfile, $snapshot) = @_;
 
     my $path = Path::Tiny->tempfile;
     $cpanfile->save($path);
-    $self->install("--installdeps", "--cpanfile", $path, ".");
+
+    my @options = ("--cpanfile", $path);
+
+    if ($snapshot) {
+        my $path = Path::Tiny->tempfile;
+        $snapshot->write_index($path);
+        push @options, "--mirror-index", $path, "--cascade-search";
+    }
+
+    $self->install("--installdeps", @options, ".");
 }
 
 sub install {
@@ -188,10 +197,17 @@ sub install {
     local $ENV{PERL_CPANM_HOME} = $dir;
     local $ENV{PERL_CPANM_OPT};
 
+    my $cpanfile = $self->try_cpanfile
+      or die "Can't locate 'cpanfile' to load module list.\n";
+
+    # one mirror for now
+    my $mirror = Module::CPANfile->load($cpanfile)->mirrors->[0];
+
     require Menlo::CLI::Compat;
 
     my $cli = Menlo::CLI::Compat->new(
         ($self->verbose ? () : "--quiet"),
+        ($mirror ? ("-M", $mirror) : ()),
         "--notest",
         "--save-dists", $self->repository_base->child('cache'),
         "-L", $self->repository_base->child('perl5'),
@@ -277,7 +293,7 @@ sub dump_bootstrap {
     }
 
     my $cpanfile = $self->try_cpanfile
-      or Carp::croak "Can't locate 'cpanfile' to load module list.";
+      or die "Can't locate 'cpanfile' to load module list.\n";
 
     my $prereqs = Module::CPANfile->load($cpanfile)->prereqs->as_string_hash;
     my $package = "Carmel::MySetup"; # hide from PAUSE
@@ -550,7 +566,7 @@ sub build_requirements {
     my $self = shift;
 
     my $cpanfile = $self->try_cpanfile
-      or Carp::croak "Can't locate 'cpanfile' to load module list.";
+      or die "Can't locate 'cpanfile' to load module list.\n";
 
     return Module::CPANfile->load($cpanfile)
       ->prereqs->merged_requirements(['runtime', 'test', 'develop'], ['requires']);
@@ -663,7 +679,7 @@ sub snapshot {
     my $self = shift;
 
     my $cpanfile = $self->try_cpanfile;
-    if (-e "$cpanfile.snapshot") {
+    if ($cpanfile && -e "$cpanfile.snapshot") {
         require Carton::Snapshot;
         my $snapshot = Carton::Snapshot->new(path => "$cpanfile.snapshot");
         $snapshot->load;
